@@ -4,6 +4,23 @@ package Circa::Indexer;
 # Copyright 2000 A.Barbet alian@alianwebserver.com.  All rights reserved.
 
 # $Log: Indexer.pm,v $
+# Revision 1.8  2000/11/23 22:11:11  Administrateur
+# - Add template as parameter
+# - Add timeout to 25s.
+# - Add updateUrl function
+# - Add "valide url" feature
+# - Add auto-categorie creation as option
+# - Add level control feature for estimate size index
+#   and limit use of speeder feature
+# - Replace count(*) with count(1)
+# - Add some function for administrate each compte
+# - Add feature to use categories a you want
+# - Add some pod documentation
+# - Correct some bugs
+# - TODO: find a bug with LinkExtor on some site like
+#         www.looksmart.bug It's give out of memory
+# 	error :-( the todo is correct this problem
+#
 # Revision 1.7  2000/10/21 15:40:10  Administrateur
 # - Remove use of modules HTML::Parse, CGI, diagnostics
 # - Correct lot of things to get down memory usage
@@ -33,7 +50,7 @@ require Exporter;
 
 @ISA = qw(Exporter);
 @EXPORT = qw();
-$VERSION = ('$Revision: 1.7 $ ' =~ /(\d+\.\d+)/)[0];
+$VERSION = ('$Revision: 1.8 $ ' =~ /(\d+\.\d+)/)[0];
 
 =head1 NAME
 
@@ -45,33 +62,26 @@ a www search engine running with Mysql
  use Circa::Indexer;
  my $indexor = new Circa::Indexer;
  
- # Connection  MySQL
  if (!$indexor->connect_mysql($user,$pass,$db)) 
 	{die "Erreur  la connection MySQL:$DBI::errstr\n";}
 	
- # Creation des tables de Circa
  $indexor->create_table_circa;
  
- # Suppression des tables de Circa
  $indexor->drop_table_circa;
  
- # Ajout d'un site
  $indexor->addSite("http://www.alianwebserver.com/",
                    'alian@alianwebserver.com',
                    "Alian Web Server");
                    
- # Indexation (mots et url) des liens non encore pars sur le premier site inscrit                  
  my ($nbIndexe,$nbAjoute,$nbWords,$nbWordsGood) = $indexor->parse_new_url(1);
  print 	"$nbIndexe pages indexes,"
  	"$nbAjoute pages ajoutes,"
  	"$nbWordsGood mots indexs,"
  	"$nbWords mots lus\n";
 
- # Interroge les pages qui n'ont pas t parses depuis plus de 1 mois sur le
- # premier site inscrit
  $indexor->update(30,1);
 
-Voir galement les fichier admin.pl et admin.cgi
+Look in admin.pl,admin.cgi,admin_compte.cgi
 
 =head1 DESCRIPTION
 
@@ -92,30 +102,80 @@ This module can:
 
 =item Parse url, and so on.
 
+=item Provide routine to administrate present url
+
 =back 
 
 Remarques:
- - This file are not added : doc,zip,ps,gif,jpg,gz,pdf,eps
- - Weight for each word is in hash $ConfigMoteur
+
+  - This file are not added : doc,zip,ps,gif,jpg,gz,pdf,eps,png,
+    deb,xls,ppt,class,GIF,css,js,wav,mid
+  - Weight for each word is in hash $ConfigMoteur
+
+=head2 How it's work ?
+
+Circa parse html document. convert it to text. It count all
+word found and put result in hash key. In addition of that, 
+it read title, keywords, description and add a weight to 
+all word found.
+
+Example:
+
+ my %ConfigMoteur=(
+        'facteur_keyword'       =>15,
+        'facteur_description'   =>10,
+        'facteur_titre'         =>10,
+        'facteur_full_text'     =>1,
+        'niveau_max'		=>7,
+        'nb_min_mots'           =>2
+        );
+
+ <html>
+ <head>
+ <meta name="KeyWords"
+ CONTENT="informatique,computing,javascript,CGI,perl">
+ <meta name="Description" CONTENT="Rubriques Informatique (Internet,Java,Javascript, CGI, Perl)">
+ <title>Alian Web Server:Informatique,Socit,Loisirs,Voyages,Expression</title>
+ </head>
+ <body>
+ different word: cgi, perl, cgi
+ </body>
+ </html>
+
+After parsing I've a hash with that:
+
+ $words{'informatique'}= 15 + 10 + 10 =35
+ $words{'cgi'} = 15 + 10 +1
+ $words{'different'} = 1
+
+Words is add to database if total found is > $ConfigMoteur{'nb_min_mots'} 
+(2 by default). But if you set to 1, database will grow very quicly but 
+allow you to perform very exact search with many worlds so you can do phrase
+searches. But if you do that, think to take a look at size of table
+relation. 
+
+After page is read, it's look into html link. And so on. At each time, the level
+grow to one. So if < to $Config{'niveau_max'}, url is added. 
 
 =head1 VERSION
 
-$Revision: 1.7 $
+$Revision: 1.8 $
 
 =cut
 
 ########## CONFIG  ##########
-my $temporate = 1;# Temporise les requetes sur le serveur d'une minute. 
-		  # A modifier que si vous tes l'administrateur de votre serveur !
-my $indexCgi = 0; # Suite les diffrents liens des CGI (ex: ?nom=toto&riri=eieiei)
-my $author = 'alian@alianwebserver.com'; # Responsable du moteur
 my %ConfigMoteur=(
-	'facteur_keyword'	=>15,
-	'facteur_description'	=>10,
-	'facteur_titre'		=>10,
-	'facteur_full_text'	=>1,
-	'nb_min_mots'		=>2
-	); # Poid des differentes parties d'un doc HTML
+	'author'		=> 'alian@alianwebserver.com', # Responsable du moteur
+	'temporate' 		=> 1,  # Temporise les requetes sur le serveur de 8s.
+	'facteur_keyword'	=> 15, # <meta name="KeyWords"
+	'facteur_description'	=> 10, # <meta name="description"
+	'facteur_titre'		=> 10, # <title></title>
+	'facteur_full_text'	=> 1,  # reste
+	'nb_min_mots'		=> 2,  # facteur min pour garder un mot
+	'niveau_max'		=> 7,  # Niveau max  indexer
+	'indexCgi'		=> 0,  # Suite les diffrents liens des CGI (ex: ?nom=toto&riri=eieiei)
+	); 
+# Mots a ne pas indexer
 my %bad = map {$_ => 1} qw (
 	le la les et des de and or the un une ou qui que quoi a an 
 	&quot je tu il elle nous vous ils elles eux ce cette ces cet celui celle ceux 
@@ -146,7 +206,7 @@ Get or set size max of file read by indexer (For avoid memory pb).
 
 =cut
 
-sub size_max($size)
+sub size_max
 	{
 	my $self = shift;
 	if (@_) {$self->{SIZE_MAX}=shift;}
@@ -160,14 +220,15 @@ sub size_max($size)
 sub set_agent
 	{
 	my ($self,$locale,$bytes)=@_;
-	if (($temporate) && (!$locale))
+	if (($ConfigMoteur{'temporate'}) && (!$locale))
 		{
-		$self->{AGENT} = new LWP::RobotUA 'CircaIndexer/0.1', $author;
+		$self->{AGENT} = new LWP::RobotUA 'CircaIndexer/0.2', $ConfigMoteur{'author'};
 		$self->{AGENT}->delay(10/60.0);
 		}
-	else {$self->{AGENT} = new LWP::UserAgent 'CircaIndexer/0.1', $author;}
+	else {$self->{AGENT} = new LWP::UserAgent 'CircaIndexer/0.1', $ConfigMoteur{'author'};}
 	if ($self->{PROXY}) {$self->{AGENT}->proxy(['http', 'ftp'], $self->{PROXY});}
 	$self->{AGENT}->max_size($self->size_max) if $self->size_max;
+	$self->{AGENT}->timeout(25); # Set timeout to 25s (defaut 180)
 	}
 
 =head2 port_mysql($port)
@@ -236,12 +297,12 @@ sub prefix_table
 
 =head2 connect_mysql($user,$password,$db,$server)
 
- $user     : Utilisateur MySQL
- $password : Mot de passe MySQL
+ $user     : User MySQL
+ $password : Password MySQL
  $db       : Database MySQL
- $server   : Adr IP du serveur MySQL
+ $server   : Adr IP MySQL
 
-Connecte l'application  MySQL. Retourne 1 si succes, 0 sinon
+Connect Circa to MySQL. Return 1 on succes, 0 else
 
 =cut
 
@@ -253,11 +314,17 @@ sub connect_mysql
 	return 1;
 	}
 
+=head2 close_connect
+
+Close connection to MySQL
+
+=cut
+
 sub close_connect {$_[0]->{DBH}->disconnect;}
 
-=head1 Methodes administration globales
+=head1 Administrative globales methods
 
-=head2 addSite($url,$email,$titre);
+=head2 addSite($url,$email,$titre,$categorieAuto,$cgi,$rep,$file);
 
 Ajoute le site d'url $url, responsable d'adresse mail $email  la bd de Circa
 
@@ -265,32 +332,85 @@ Ajoute le site d'url $url, responsable d'adresse mail $email  la bd de Circa
 
 sub addSite 
 	{
-	my ($self,$url,$email,$titre)=@_;
-	my $sth = $self->{DBH}->prepare("insert into ".$self->prefix_table."responsable(email,titre) values('$email','$titre')");
-	$sth->execute;
+	my ($self,$url,$email,$titre,$categorieAuto,$cgi,$rep,$file)=@_;
+	#print "$url,$email,$titre,$categorieAuto,$cgi,$rep,$file\n";
+	if ($cgi) 
+		{
+		$file=$cgi->param('file');
+		my $tmpfile=$cgi->tmpFileName($file); # chemin du fichier temp
+		if ($file=~/.*\\(.*)$/) {$file=$1;}
+		my $fileC=$file;
+		$file = $rep.$file;	
+		use File::Copy; 
+		copy($tmpfile,$file) || die "Impossible de creer $file avec $tmpfile:$!\n<br>";
+		}
+
+	if (!$categorieAuto) {$categorieAuto=0;}
+	my $sth = $self->{DBH}->prepare("insert into ".$self->prefix_table."responsable(email,titre,categorieAuto,masque) values('$email','$titre',$categorieAuto,'$file')");
+	$sth->execute || print "Erreur: $DBI::errstr<br>\n";
 	$sth->finish;
 	$self->create_table_circa_id($sth->{'mysql_insertid'});
-	$self->add_site($url,$sth->{'mysql_insertid'});
+	$self->add_site($url,$sth->{'mysql_insertid'},undef,1);
 	}
 
-=head2 addLocalSite($url,$email,$titre,$local_url,$path,$urlRacine);
+=head2 addLocalSite($url,$email,$titre,$local_url,$path,$urlRacine,$categorieAuto,$cgi,$rep,$file);
 
-Ajoute le site d'url $url, responsable d'adresse mail $email  la bd de Circa
+Add a local $url
 
 =cut
 
 sub addLocalSite 
 	{
-	my ($self,$url,$email,$titre,$local_url,$path,$urlRacine)=@_;
-	my $sth = $self->{DBH}->prepare("insert into ".$self->prefix_table."responsable(email,titre) values('$email','$titre')");
+	my ($self,$url,$email,$titre,$local_url,$path,$urlRacine,$categorieAuto,$cgi,$rep,$file)=@_;
+	if ($cgi) 
+		{
+		$file=$cgi->param('file');
+		my $tmpfile=$cgi->tmpFileName($file); # chemin du fichier temp
+		if ($file=~/.*\\(.*)$/) {$file=$1;}
+		my $fileC=$file;
+		$file = $rep.$file;	
+		use File::Copy; 
+		copy($tmpfile,$file) || die "Impossible de creer $file avec $tmpfile:$!\n<br>";
+		}
+	my $sth = $self->{DBH}->prepare("insert into ".$self->prefix_table."responsable(email,titre,categorieAuto) values('$email','$titre',$categorieAuto)");
 	$sth->execute;
 	$sth->finish;
 	my $id = $sth->{'mysql_insertid'};
 	$self->{DBH}->do("insert into ".$self->prefix_table."local_url values($id,'$urlRacine','$path');");
 	$self->create_table_circa_id($sth->{'mysql_insertid'});	
-	$self->add_site($url,$id,$local_url) || print "Erreur: $DBI::errstr<br>\n";
+	$self->add_site($url,$id,$local_url,1) || print "Erreur: $DBI::errstr<br>\n";
 	}
 
+=head2 updateUrl($compte,$id,$url,$urllocal,$titre,$description,$langue,
+		 $categorie,$browse_categorie,$parse,$valide,$niveau,$last_check,$last_update)
+
+Update url $id on table $compte
+
+=cut
+
+sub updateUrl
+	{
+	my ($self,$compte,$id,$url,$urllocal,$titre,$description,$langue,$categorie,$browse_categorie,$parse,$valide,$niveau,$last_check,$last_update)=@_;
+	$titre=~s/'/\\'/g;
+	$description=~s/'/\\'/g;	
+	my $requete ="update ".$self->prefix_table.$compte."links set 
+		url='$url',
+		local_url='$urllocal',
+		titre='$titre',
+		description='$description',
+		langue='$langue',
+		categorie=$categorie,
+		browse_categorie='$browse_categorie',
+		parse='$parse',
+		valide=$valide,
+		niveau=$niveau,
+		last_check='$last_check',
+		last_update='$last_update'
+	where id=$id";#print $requete;
+	$self->{DBH}->do($requete) || print "Erreur: $requete $DBI::errstr<br>\n";
+	}
+	
+
 =head2 parse_new_url($idp)
 
 Parse les pages qui viennent d'tre ajoute. Le programme va analyser toutes
@@ -307,28 +427,30 @@ sub parse_new_url
 	#system('/bin/ps m|grep perl>>/tmp/ressource');
 	my $idp=$_[0];
 	my ($nb,$nbAjout,$nbWords,$nbWordsGood)=(0,0,0,0);
-	my $requete="select id,url from ".$self->prefix_table.$idp."links where parse='0' and local_url is null order by id";
+	my $requete="select id,url,niveau,categorie from ".$self->prefix_table.$idp."links where parse='0' and valide=1 and local_url is null order by id";
 	my $sth = $self->{DBH}->prepare($requete);
 	$self->set_agent;
 	if ($sth->execute())
 		{
-		while (my ($id,$url)=$sth->fetchrow_array)
-			{
-			my ($res,$nbw,$nbwg) = $self->look_at($url,$id,$idp,undef,undef);
+		my ($categorieAuto)=$self->get_first("select categorieAuto from ".$self->prefix_table."responsable where id=$idp");
+		while (my ($id,$url,$niveau,$categorie)=$sth->fetchrow_array)
+			{			
+			my ($res,$nbw,$nbwg) = $self->look_at($url,$id,$idp,undef,undef,$categorieAuto,$niveau,$categorie);
 			if ($res==-1) {$self->{DBH}->do("delete from ".$self->prefix_table.$idp."links where id=$id");}
 			else {$nbAjout+=$res;$nbWords+=$nbw;$nb++;$nbWordsGood+=$nbwg;}
 			}
 		}
 	else {print "\nYou must call this->create_table_circa before calling this method.\n";}
 	$sth->finish;
-	$requete="select id,url,local_url from ".$self->prefix_table.$idp."links where parse='0' and local_url is not null order by id";
+	$requete="select id,url,local_url,niveau,categorie from ".$self->prefix_table.$idp."links where parse='0' and valide=1 and local_url is not null order by id";
 	$sth = $self->{DBH}->prepare($requete);
 	$self->set_agent('local');
 	if ($sth->execute())
 		{
-		while (my ($id,$url,$local_url)=$sth->fetchrow_array)
-			{
-			my ($res,$nbw,$nbwg) = $self->look_at($url,$id,$idp,undef,$local_url);
+		my ($categorieAuto)=$self->get_first("select categorieAuto from ".$self->prefix_table."responsable where id=$idp");
+		while (my ($id,$url,$local_url,$niveau,$categorie)=$sth->fetchrow_array)
+			{			
+			my ($res,$nbw,$nbwg) = $self->look_at($url,$id,$idp,undef,$local_url,$categorieAuto,$niveau,$categorie);
 			if ($res==-1) {$self->{DBH}->do("delete from ".$self->prefix_table.$idp."links where id=$id");}
 			else {$nbAjout+=$res;$nbWords+=$nbw;$nb++;$nbWordsGood+=$nbwg;}
 			}
@@ -350,26 +472,28 @@ sub update
 	my ($this,$xj,$idp)=@_;
 	$this->set_agent;
 	my ($nb,$nbAjout,$nbWords,$nbWordsGood)=(0,0,0,0);
-	my $requete="select id,url,UNIX_TIMESTAMP(last_update) from ".$this->prefix_table.$idp."links where TO_DAYS(NOW()) >= (TO_DAYS(last_check) + $xj) and local_url is null order by url";
+	my $requete="select id,url,UNIX_TIMESTAMP(last_update),niveau,categorie from ".$this->prefix_table.$idp."links where TO_DAYS(NOW()) >= (TO_DAYS(last_check) + $xj) and local_url is null order by url";
 	my $sth = $this->{DBH}->prepare($requete);
 	if ($sth->execute())
 		{
-		while (my ($id,$url,$last_update)=$sth->fetchrow_array)
+		while (my ($id,$url,$last_update,$niveau,$categorie)=$sth->fetchrow_array)
 			{
-			my ($res,$nbw,$nbwg) = $this->look_at($url,$id,$idp,$last_update,undef);
+			my ($categorieAuto)=$this->get_first("select categorieAuto from ".$this->prefix_table."responsable where id=$idp");
+			my ($res,$nbw,$nbwg) = $this->look_at($url,$id,$idp,$last_update,undef,$categorieAuto,$niveau,$categorie);
 			if ($res==-1) {$this->{DBH}->do("delete from ".$this->prefix_table.$idp."links where id=$id");}
 			else {$nbAjout+=$res;$nbWords+=$nbw;$nb++;$nbWordsGood+=$nbwg;}
 			}
 		}
 	else {print "\nYou must call this->create_table_circa before calling this method.\n";}
 	$sth->finish;	
-	$requete="select id,url,UNIX_TIMESTAMP(last_update),local_url from ".$this->prefix_table.$idp."links where TO_DAYS(NOW()) >= (TO_DAYS(last_check) + $xj) and local_url is not null order by url";
+	$requete="select id,url,UNIX_TIMESTAMP(last_update),local_url,niveau,categorie from ".$this->prefix_table.$idp."links where TO_DAYS(NOW()) >= (TO_DAYS(last_check) + $xj) and local_url is not null order by url";
 	$sth = $this->{DBH}->prepare($requete);
 	if ($sth->execute())
 		{
-		while (my ($id,$url,$last_update,$local_url)=$sth->fetchrow_array)
+		while (my ($id,$url,$last_update,$local_url,$niveau,$categorie)=$sth->fetchrow_array)
 			{
-			my ($res,$nbw,$nbwg) = $this->look_at($url,$id,$idp,$last_update,$local_url);
+			my ($categorieAuto)=$this->get_first("select categorieAuto from ".$this->prefix_table."responsable where id=$idp");
+			my ($res,$nbw,$nbwg) = $this->look_at($url,$id,$idp,$last_update,$local_url,$categorieAuto,$niveau,$categorie);
 			if ($res==-1) {$this->{DBH}->do("delete from ".$this->prefix_table.$idp."links where id=$id");}
 			else {$nbAjout+=$res;$nbWords+=$nbw;$nb++;$nbWordsGood+=$nbwg;}
 			}
@@ -399,6 +523,8 @@ CREATE TABLE ".$self->prefix_table."responsable (
    id 		int(11) DEFAULT '0' NOT NULL auto_increment,
    email	char(25) NOT NULL,
    titre	char(50) NOT NULL,
+   categorieAuto tinyint DEFAULT '0' NOT NULL,
+   masque	char(150) NOT NULL,
    PRIMARY KEY (id)   
 )";
 
@@ -484,20 +610,23 @@ CREATE TABLE ".$self->prefix_table.$id."links (
    id 		int(11) DEFAULT '0' NOT NULL auto_increment,
    url 		varchar(255) NOT NULL,
    local_url 	varchar(255),
-   titre 	tinyblob NOT NULL,
-   description 	tinyblob NOT NULL,
+   titre 	varchar(255) NOT NULL,
+   description 	blob NOT NULL,
    langue 	char(6) NOT NULL,
-   valide 	tinyint(1) DEFAULT '0' NOT NULL,
-   categorie 	int(11),
+   valide 	tinyint DEFAULT '0' NOT NULL,
+   categorie 	int(11) DEFAULT '0' NOT NULL,
    last_check 	datetime DEFAULT '0000-00-00' NOT NULL,
    last_update  datetime DEFAULT '0000-00-00' NOT NULL,
    parse 	ENUM('0','1') DEFAULT '0' NOT NULL,
+   browse_categorie ENUM('0','1') DEFAULT '0' NOT NULL,
+   niveau 	tinyint DEFAULT '0' NOT NULL,
    PRIMARY KEY (id),
    KEY id (id),
    UNIQUE id_2 (id),
    KEY id_3 (id),
    KEY url (url),
-   UNIQUE url_2 (url)
+   UNIQUE url_2 (url),
+   KEY categorie (categorie)
 )";
 	$self->{DBH}->do($requete) || print $DBI::errstr,"<br>\n";
 
@@ -529,15 +658,28 @@ Affiche le debut de document (<head></head>)
 
 sub start_classic_html
 	{
-	return start_html(
+	my ($self,$cgi)=@_;
+	return $cgi->start_html(
 		-'title'	=> 'Circa',
 		-'author'	=> 'alian@alianwebserver.com',
 		-'meta'		=> {'keywords'=>'circa,recherche,annuaire,moteur',
         	-'copyright'	=> 'copyright 1997-2000 AlianWebServer'},
-		-'style'	=> {'src'=>"circa.css"},
 		-'dtd'		=> '-//W3C//DTD HTML 4.0 Transitional//EN" "http://www.w3.org/TR/REC-html40/loose.dtd')."\n";
 	}
 
+sub header_compte
+	{
+	my ($self,$cgi,$compte,$script)=@_;
+	my $buf='<ul>'."\n";
+	$buf.=$cgi->li("<p class=\"without-margin\"><a href=\"$script?compte=$compte\">Infos gnrales</a></p>")."\n";
+	$buf.=$cgi->li("<p class=\"without-margin\"><a href=\"$script?compte=$compte&ecran_stats=1\">Statistiques</a></p>")."\n";
+	$buf.=$cgi->li("<p class=\"without-margin\"><a href=\"$script?compte=$compte&ecran_urls=1\">Gestion des url</a></p>")."\n";
+	$buf.=$cgi->li("<p class=\"without-margin\"><a href=\"$script?compte=$compte&ecran_validation=1\">Validation des url</a></p>")."\n";
+	$buf.=$cgi->li("<p class=\"without-margin\"><a href=\"$script?compte=$compte&ecran_categorie=1\">Gestion des categories</a></p>")."\n";	
+	$buf.='</ul>'."\n";
+	return $buf;
+	}
+
 =head2 get_liste_liens($id)
 
 Rend un buffer contenant une balise select initialise avec les donnes 
@@ -547,8 +689,7 @@ de la table links responsable $id
 
 sub get_liste_liens
 	{
-	my $self=shift;
-	my ($id) =@_;
+	my ($self,$id,$cgi)=@_;
 	my %tab;
 	my $sth = $self->{DBH}->prepare("select id,url from ".$self->prefix_table.$id."links");
 	$sth->execute() || print &header,$DBI::errstr,"<br>\n";	
@@ -561,12 +702,41 @@ sub get_liste_liens
 		}	
 	$sth->finish;
 	my @l =sort { $tab{$a} cmp $tab{$b} } keys %tab;
-	return scrolling_list(	-'name'=>'id',
+	return $cgi->scrolling_list(	-'name'=>'id',
                		        -'values'=>\@l,
                		        -'size'=>1,
                        		-'labels'=>\%tab);
         }
 
+
+=head2 get_liste_liens_a_valider($id)
+
+Rend un buffer contenant une balise select initialise avec les donnes 
+de la table links responsable $id liens non valides
+
+=cut
+
+sub get_liste_liens_a_valider
+	{
+	my ($self,$id,$cgi)=@_;
+	my (%tab,$buf);
+	$buf='<table>';
+	my $sth = $self->{DBH}->prepare("select id,url from ".$self->prefix_table.$id."links where valide=0");
+	$sth->execute() || print &header,$DBI::errstr,"<br>\n";	
+	while (my @row=$sth->fetchrow_array) 
+		{
+		$self->set_host_indexed($row[1]);
+		my $racine=$self->host_indexed;
+		$tab{$row[0]}=$row[1];
+		$tab{$row[0]}=~s/www\.//g;
+		}	
+	$sth->finish;
+	my @l =sort { $tab{$a} cmp $tab{$b} } keys %tab;
+	foreach (@l)  {$buf.=$cgi->Tr($cgi->td("<input onClick=\"document.reno.submit();return true;\" type=\"radio\" name=\"id_valide_url\" value=\"$_\">"),$cgi->td("<a target=_blank href=\"$tab{$_}\">$tab{$_}</a>"))."\n";}
+	$buf.='</table>';
+	return $buf;
+        }
+
 =head2 get_liste_site
 
 Rend un buffer contenant une balise select initialise avec les donnes 
@@ -576,17 +746,17 @@ de la table responsable
 
 sub get_liste_site
 	{
-	my $self=shift;
+	my ($self,$cgi)=@_;
 	my %tab;
 	my $sth = $self->{DBH}->prepare("select id,email,titre from ".$self->prefix_table."responsable");
 	$sth->execute() || print &header,$DBI::errstr,"<br>\n";
 	while (my @row=$sth->fetchrow_array) {$tab{$row[0]}="$row[1]/$row[2]";}	
 	$sth->finish;
 	my @l =sort { $tab{$a} cmp $tab{$b} } keys %tab;
-	return scrolling_list(	-'name'=>'id',
-               		        -'values'=>\@l,
-               		        -'size'=>1,
-                       		-'labels'=>\%tab);
+	return $cgi->scrolling_list(	-'name'=>'id',
+               		        	-'values'=>\@l,
+               		        	-'size'=>1,
+                       			-'labels'=>\%tab);
         }
 
 =head2 get_liste_categorie($id)
@@ -598,25 +768,87 @@ de la table categorie responsable $id
 
 sub get_liste_categorie
 	{
-	my $self=shift;
-	my ($id) =@_;
-	my (%tab,%tab2);
+	my ($self,$id,$cgi,$null,$nom,$valeur)=@_;
+	my (%tab,%tab2,$erreur);
 	my $sth = $self->{DBH}->prepare("select id,nom,parent from ".$self->prefix_table.$id."categorie");
-	$sth->execute() || print &header,$DBI::errstr,"<br>\n";	
+	$sth->execute() || return;
 	while (my @row=$sth->fetchrow_array) {$tab2{$row[0]}[0]=$row[1];$tab2{$row[0]}[1]=$row[2];}	
 	$sth->finish;
 	foreach my $key (keys (%tab2)) 
 		{
-		my ($nb)=$self->get_first("select count(*) from ".$self->prefix_table.$id."links where categorie=$key");
+		my ($nb)=$self->get_first("select count(1) from ".$self->prefix_table.$id."links where categorie=$key");
 		$tab{$key}= getParent($key,%tab2)." ($nb)";
 		}
-
 	my @l =sort { $tab{$a} cmp $tab{$b} } keys %tab;
-	return scrolling_list(	-'name'=>'id',
-               		        -'values'=>\@l,
-               		        -'size'=>1,
-                       		-'labels'=>\%tab);
-        }
+	if ($null) 
+		{
+		my ($nb)=$self->get_first("select count(1) from ".$self->prefix_table.$id."links where categorie=0");
+		$tab{"0"}=" ($nb)";unshift(@l,0);
+	}
+	$nom = 'id' if (!$nom);
+	if ($valeur) 
+		{
+		return $cgi->scrolling_list(-'name'=>$nom,
+               			        -'values'=>\@l,
+               			        -'default'=>$valeur,
+               			        -'size'=>1,
+                       			-'labels'=>\%tab);		
+		}
+	else 
+		{
+		return $cgi->scrolling_list(-'name'=>$nom,
+               			        -'values'=>\@l,
+               			        -'size'=>1,
+                       			-'labels'=>\%tab);
+                }    
+       }
+
+sub get_liste_mot
+	{
+	my ($self,$compte,$id)=@_;
+	my @l;
+	my $sth = $self->{DBH}->prepare("select mot from ".$self->prefix_table.$compte."relation where id_site=$id");
+	$sth->execute() || print "Erreur: $DBI::errstr\n";		
+	while (my ($l)=$sth->fetchrow_array) {push(@l,$l);}
+	return join(' ',@l);	
+	}
+
+sub get_liste_langues
+	{
+	my ($self,$id,$valeur,$cgi)=@_;
+	my @l;
+	my $sth = $self->{DBH}->prepare("select distinct langue from ".$self->{PREFIX_TABLE}.$id."links");
+	$sth->execute() || print "Erreur: $DBI::errstr\n";		
+	while (my ($l)=$sth->fetchrow_array) {push(@l,$l);}
+	$sth->finish;
+	my %langue=(	
+		'unkno'=>'unkno',
+  		'da'=>'Dansk',
+		'de'=>'Deutsch',
+		'en'=>'English',
+		'eo'=>'Esperanto',
+  		'es'=>'Espanl',
+  		'fi'=>'Suomi', 
+		'fr'=>'Francais',  		
+  		'hr'=>'Hrvatski',
+  		'hu'=>'Magyar',	
+		'it'=>'Italiano',
+    		'nl'=>'Nederlands',
+  		'no'=>'Norsk',
+  		'pl'=>'Polski', 
+    		'pt'=>'Portuguese', 
+  		'ro'=>'Romn', 
+    		'sv'=>'Svenska', 
+  		'tr'=>'TurkCe', 
+		'0'=>'All'
+		);
+	my $scrollLangue =  
+		$cgi->scrolling_list(	-'name'=>'langue',
+               			        -'values'=>\@l,
+               		        	-'size'=>1,
+	                       		-'default'=>$valeur,
+        	               		-'labels'=>\%langue);
+	}
 
 =head2 fill_template($masque,$vars)
 
@@ -640,7 +872,11 @@ sub fill_template
 	open(FILE,$masque) || die "Can't read $masque<br>";
 	my @buf=<FILE>;
 	close(FILE);
-	while (my ($n,$v)=each(%$vars)) {if ($v) {map {s/<\? \$$n \?>/$v/gm} @buf;}}	
+	while (my ($n,$v)=each(%$vars)) 
+		{
+		if (defined($v)) {map {s/<\? \$$n \?>/$v/gm} @buf;}
+		else {map {s/<\? \$$n \?>//gm} @buf;}
+		}	
 	return join('',@buf);
 	}
 
@@ -666,10 +902,10 @@ sub admin_compte
 	my ($responsable,$titre) = $self->get_first("select email,titre from ".$self->prefix_table."responsable where id=$compte");
 	my ($racine) 		 = $self->get_first("select min(id) from ".$self->prefix_table.$compte."links");
 	($racine) 		 = $self->get_first("select url from ".$self->prefix_table.$compte."links where id=$racine");
-	my ($nb_page) 		 = $self->get_first("select count(*) from ".$self->prefix_table.$compte."links");
+	my ($nb_page) 		 = $self->get_first("select count(1) from ".$self->prefix_table.$compte."links");
 	my ($last_index)	 = $self->get_first("select max(last_check) from ".$self->prefix_table.$compte."links");
-	my ($nb_requetes) 	 = $self->get_first("select count(*) from ".$self->prefix_table.$compte."stats");
-	my ($nb_words) 		 = $self->get_first("select count(*) from ".$self->prefix_table.$compte."relation");
+	my ($nb_requetes) 	 = $self->get_first("select count(1) from ".$self->prefix_table.$compte."stats");
+	my ($nb_words) 		 = $self->get_first("select count(1) from ".$self->prefix_table.$compte."relation");
 	return ($responsable,$titre,$nb_page,$nb_words,$last_index,$nb_requetes,$racine);
 	}
 
@@ -684,7 +920,7 @@ sub most_popular_word
 	{
 	my $self = shift;
 	my %l;
-	my $requete = "select mot,count(*) from ".$self->prefix_table.$_[1]."relation r group by r.mot order by 2 desc limit 0,$_[0]";
+	my $requete = "select mot,count(1) from ".$self->prefix_table.$_[1]."relation r group by r.mot order by 2 desc limit 0,$_[0]";
 	my $sth = $self->{DBH}->prepare($requete);	
 	$sth->execute;
 	while (my ($word,$nb)=$sth->fetchrow_array) {$l{$word}=$nb;}
@@ -705,6 +941,18 @@ sub delete_url
 	$this->{DBH}->do("delete from ".$this->prefix_table.$compte."links where id = $id_url");
 	}
 
+=head2 valide_url($compte,$id_url)
+
+Supprime le lien $id_url de la table $compte/relation et $compte/links
+
+=cut
+
+sub valide_url
+	{
+	my ($this,$compte,$id_url)=@_;
+	$this->{DBH}->do("update ".$this->prefix_table.$compte."links set valide=1 where id = $id_url");	
+	}
+
 =head2 delete_categorie($compte,$id)
 
 Supprime la categorie $id pour le compte de responsable $compte et 
@@ -737,6 +985,18 @@ sub rename_categorie
 	$this->{DBH}->do("update ".$this->prefix_table.$compte."categorie set nom='$nom' where id = $id")|| print "Erreur:$DBI::errstr<br>\n";
 	}
 
+=head2 deplace_categorie($compte,$id1,$id2)
+
+Move url from one categorie to another
+
+=cut
+
+sub deplace_categorie
+	{
+	my ($this,$compte,$id1,$id2)=@_;
+	$this->{DBH}->do("update ".$this->prefix_table.$compte."links set categorie=$id2 where categorie = $id1")|| print "Erreur:$DBI::errstr<br>\n";
+	}
+
 =head2 inscription($email,$url,$titre)
 
 Inscrit un site dans une table temporaire
@@ -766,11 +1026,11 @@ que le nombre de mots trouves sinon.
 
 sub look_at
 	{
-	my($this,$url,$idc,$idr,$lastModif,$url_local) = @_;
+	my($this,$url,$idc,$idr,$lastModif,$url_local,$categorieAuto,$niveau,$categorie) = @_;
 	my ($l,$url_orig,$racineFile,$racineUrl,$lastUpdate);
 	if ($url_local) 
 		{
-		$temporate=0;
+		$ConfigMoteur{'temporate'}=0;
 		if ($url_local=~/.*\/$/) 
 			{
 			chop($url_local);
@@ -812,8 +1072,9 @@ sub look_at
 			# Titre
 			my $titre = $res->title || $url;
 			$titre=~s/'/\\'/g if ($titre);
-			# Categorie
-			my $categorie = $this->get_categorie($url,$idr);
+			# Categorie			
+			if ($categorieAuto) {$categorie = $this->get_categorie($url,$idr);}
+			if (!$categorie) {$categorie=0;}
 			my $requete ="
 				update 	".$this->prefix_table.$idr."links 
 				set 	parse		= '1',
@@ -848,6 +1109,12 @@ sub look_at
 				}
 			my $nbw=keys %$l;undef(%$l);
 			
+			# On n'indexe pas les liens si on est au niveau max
+			if ($niveau > $ConfigMoteur{'niveau_max'})
+				{
+				print "Niveau max atteint. Liens suivants de cette page ignors<br>\n";
+				return (0,0,0);	
+				}
 	  		# Traitement des url trouves
 			$analyseur->parse($res->content);
 			my @l = $analyseur->links;
@@ -859,9 +1126,9 @@ sub look_at
 					my $urlb = $$var[2];
 					$urlb=~s/$racineFile/$racineUrl/g;
 					#print h1("Ajout site local:$$var[2] pour $racineFile");
-					$this->add_site($urlb,$idr,$$var[2]) && $nburl++;
+					$this->add_site($urlb,$idr,$$var[2],undef,$niveau+1,$categorie) && $nburl++;
 					}
-				elsif ($$var[2]) {$this->add_site($$var[2],$idr);$nburl++;}
+				elsif ($$var[2]) {$this->add_site($$var[2],$idr,undef,undef,$niveau+1,$categorie);$nburl++;}
 				}			
 			return ($nburl,$nbw,$nbwg);
 			}
@@ -872,7 +1139,7 @@ sub look_at
 	else {print "Url non valide:$url\n";return (-1,0,0);} 	
 	}
 
-=head2 add_site($url,$idMan)
+=head2 add_site($url,$idMan,[$local_url],[$browse_categorie],[$niveau],[$categorie])
 
 Ajoute un site  la table links. 
 
@@ -883,17 +1150,17 @@ Ajoute un site  la table links.
 
 sub add_site 
 	{
-	my ($self,$url,$idMan,$local_url)=@_;	
-	if ($local_url) 
-		{
-		$self->{DBH}->do("insert into ".$self->prefix_table.$idMan."links(url,titre,description,local_url)
-					 values ('$url',' ',' ','$local_url')") || return 0;
-		}
-	else 
-		{
-		$self->{DBH}->do("insert into ".$self->prefix_table.$idMan."links(url,titre,description) 
-					 values ('$url',' ',' ')") || return 0;
-		}
+	my ($self,$url,$idMan,$local_url,$browse_categorie,$niveau,$categorie)=@_;		
+	if (!$niveau) {$niveau=0;}
+	my ($req1,$req2) = ("insert into ".$self->prefix_table.$idMan."links(url,titre,description,niveau,valide",
+			    "values ('$url',' ',' ',$niveau,1");
+	if ($local_url) {$req1.=',local_url';$req2.=",'$local_url'";}
+	if ($browse_categorie) {$req1.=',browse_categorie';$req2.=",'1'";}
+	if ($categorie) {$req1.=',categorie';$req2.=",$categorie";}	
+	$req1.=') ';$req2.=")";
+	#print "requete:$req1 $req2<br>";
+	$self->{DBH}->do($req1.$req2) || return 0; #print &header,$req1,$req2,$DBI::errstr,"<br>\n";	
+	print "Add $url<br>\n";
 	return 1;
 	}
 
@@ -1039,11 +1306,11 @@ sub check_links
 	{
      	my($self,$tag,$links) = @_;
      	my $host = $self->host_indexed;
-     	my $bad = qr/\.(doc|zip|ps|gif|jpg|gz|pdf|eps|png|deb|xls|ppt|class|GIF|css|js|wav)$/i;
+     	my $bad = qr/\.(doc|zip|ps|gif|jpg|gz|pdf|eps|png|deb|xls|ppt|class|GIF|css|js|wav|mid)$/i;
 	if (($tag) && ($links) && ($tag eq 'a') && ($links=~/^$host/) && ($links !~ $bad)) 
 		{
 		if ($links=~/^(.*?)#/) {$links=$1;} # Don't add anchor
-		if ((!$indexCgi)&&($links=~/^(.*?)\?/)) {$links=$1;}		
+		if ((!$ConfigMoteur{'indexCgi'})&&($links=~/^(.*?)\?/)) {$links=$1;}		
 		return $links;		
 		}
 	return 0;
